//+------------------------------------------------------------------+
//|                                                MTF TMA Slope RSI |
//|                      Copyright 2012, Deltabron - Paul Geirnaerdt |
//|                                          http://www.deltabron.nl |
//+------------------------------------------------------------------+
#property copyright "Copyright 2013, Deltabron - Paul Geirnaerdt"
#property link      "http://www.deltabron.nl"

#property indicator_separate_window
#property indicator_buffers 8

#property indicator_minimum 0
#property indicator_maximum 100
#property indicator_color1 SlateGray 
#property indicator_levelstyle STYLE_SOLID
#property indicator_level1 20
#property indicator_level2 80

#define version            "v-Renko"

//+------------------------------------------------------------------+
//| Release Notes                                                    |
//+------------------------------------------------------------------+

extern string  gen               = "----General inputs----";
extern string  nonPropFont       = "Lucida Console";
extern int     fontSize          = 14;

extern string  ind               = "----Indicator inputs----";
extern int     maxBars           = 200;
extern int     lineWidth         = 3;
extern int     RSIPeriod         = 2;
extern bool    RenkoChart        = true;
extern bool    showW1            = false;
extern bool    showD1            = false;
extern bool    showH4            = false;
extern bool    showH1            = false;
extern bool    showM30           = false;
extern bool    showM15           = false;
extern bool    showM5            = false;
extern bool    showM1            = false;
extern string  ind_sm            = "----Add Sunday candles to Monday?----";
extern bool    addSundayToMonday = true;

extern string  col               = "----Colo(u)rs----";
extern color   colorW1           = C'0xFF,0x00,0x00';
extern color   colorD1           = C'0xFF,0x44,0x44';
extern color   colorH4           = Gold;
extern color   colorH1           = C'0xFF,0xCC,0xCC';
extern color   colorM30          = C'0x00,0xFF,0x00';
extern color   colorM15          = C'0x44,0xFF,0x44';
extern color   colorM5           = C'0x88,0xFF,0x88';
extern color   colorM1           = C'0x44,0x44,0xFF';

//---- buffers
double W1Buffer[];
double D1Buffer[];
double H4Buffer[];
double H1Buffer[];
double M30Buffer[];
double M15Buffer[];
double M5Buffer[];
double M1Buffer[];

//-------buffer for RSICalc
double TMA_Slope_Values[];

//---- global variables
string   indicatorName = "RSI-TMA";
string   shortName;
datetime currentTime;
string   almostUniqueIndex;
bool     sundayCandlesDetected;                          
int      userTimeFrame;
int      limit;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   limit = maxBars;

   ArrayResize( TMA_Slope_Values, limit );
   ArraySetAsSeries( TMA_Slope_Values, true );

   shortName = indicatorName + " - " + version;
   IndicatorShortName(shortName);

   SetIndexBuffer ( 0, W1Buffer );
   SetIndexBuffer ( 1, D1Buffer );
   SetIndexBuffer ( 2, H4Buffer );
   SetIndexBuffer ( 3, H1Buffer );
   SetIndexBuffer ( 4, M30Buffer );
   SetIndexBuffer ( 5, M15Buffer );
   SetIndexBuffer ( 6, M5Buffer );
   SetIndexBuffer ( 7, M1Buffer );

   SetIndexLabel ( 0, "W1" );
   SetIndexLabel ( 1, "D1" );
   SetIndexLabel ( 2, "H4" );
   SetIndexLabel ( 3, "H1" );
   SetIndexLabel ( 4, "M30" );
   SetIndexLabel ( 5, "M15" );
   SetIndexLabel ( 6, "M5" );
   SetIndexLabel ( 7, "M1" );
   
   sundayCandlesDetected = false;
   for ( int i = 0; i < 8; i++ )
   {
      if ( TimeDayOfWeek( iTime( NULL, PERIOD_D1, i ) ) == 0 )
      {
         sundayCandlesDetected = true;
         break;
      }
   }
   
   string now = TimeCurrent();
   almostUniqueIndex = StringSubstr(now, StringLen(now) - 3);
   
   return ( 0 );
}

//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   int windex = WindowFind ( shortName );
   if ( windex > 0 )
   {
      ObjectsDeleteAll ( windex );
   }   

   return ( 0 );
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   SetIndexStyle ( 0, DRAW_LINE, STYLE_SOLID, lineWidth, colorW1 );
   SetIndexStyle ( 1, DRAW_LINE, STYLE_SOLID, lineWidth, colorD1 );
   SetIndexStyle ( 2, DRAW_LINE, STYLE_SOLID, lineWidth, colorH4 );
   SetIndexStyle ( 3, DRAW_LINE, STYLE_SOLID, lineWidth, colorH1 );
   SetIndexStyle ( 4, DRAW_LINE, STYLE_SOLID, lineWidth, colorM30 );
   SetIndexStyle ( 5, DRAW_LINE, STYLE_SOLID, lineWidth, colorM15 );
   SetIndexStyle ( 6, DRAW_LINE, STYLE_SOLID, lineWidth, colorM5 );
   SetIndexStyle ( 7, DRAW_LINE, STYLE_SOLID, lineWidth, colorM1 );

   int i;
   int counted_bars = IndicatorCounted();

   if ( counted_bars < 0 ) return ( -1 );
   if ( counted_bars > 0 ) counted_bars--;
   limit = Bars - counted_bars;
   if ( maxBars > 0 && limit > maxBars ) limit = maxBars;

   for ( i = 0; i < limit; i++ )
   {
      currentTime = Time[i];
      
      if ( RenkoChart ) W1Buffer[i] = GetSlopeRSI( Symbol(), Period(), iBarShift ( NULL,Period(), currentTime ) );
      if ( showW1 ) W1Buffer[i] = GetSlopeRSI( Symbol(), PERIOD_W1, iBarShift ( NULL, PERIOD_W1, currentTime ) );
      if ( showD1 ) D1Buffer[i] = GetSlopeRSI( Symbol(), PERIOD_D1, iBarShift ( NULL, PERIOD_D1, currentTime ) );
      if ( showH4 ) H4Buffer[i] = GetSlopeRSI( Symbol(), PERIOD_H4, iBarShift ( NULL, PERIOD_H4, currentTime ) );
      if ( showH1 ) H1Buffer[i] = GetSlopeRSI( Symbol(), PERIOD_H1, iBarShift ( NULL, PERIOD_H1, currentTime ) );
      if ( showM30 ) M30Buffer[i] = GetSlopeRSI( Symbol(), PERIOD_M30, iBarShift ( NULL, PERIOD_M30, currentTime ) );
      if ( showM15 ) M15Buffer[i] = GetSlopeRSI( Symbol(), PERIOD_M15, iBarShift ( NULL, PERIOD_M15, currentTime ) );
      if ( showM5 ) M5Buffer[i] = GetSlopeRSI( Symbol(), PERIOD_M5, iBarShift ( NULL, PERIOD_M5, currentTime ) );
      if ( showM1 ) M1Buffer[i] = GetSlopeRSI( Symbol(), PERIOD_M1, iBarShift ( NULL, PERIOD_M1, currentTime ) );
       
   }
   return ( 0 );
}


//+------------------------------------------------------------------+
//| GetSlopeRSI()                                                                         |
//+------------------------------------------------------------------+
double GetSlopeRSI(string symbol, int tf, int shift)
{
   double slope[];
   int workPeriod = 17;                                         // RSI period Bob's default = 2, + overhead
   ArrayResize( slope, workPeriod );
   ArraySetAsSeries( slope, true );
   for ( int i = 0; i < workPeriod; i++ )
   {
      slope[i] = GetSlope( symbol, tf, shift + i );
   }
   return( iRSIOnArray( slope, workPeriod, 2, 0 ) );    // Again, 2 is Bob's default
}

//+------------------------------------------------------------------+
//| GetSlope()                                                                              |
//+------------------------------------------------------------------+
double GetSlope(string symbol, int tf, int shift)
{
   int shiftWithoutSunday = shift;
   if ( addSundayToMonday && sundayCandlesDetected && tf == PERIOD_D1 )
   {
      if ( TimeDayOfWeek( iTime( symbol, PERIOD_D1, shift ) ) == 0  ) shiftWithoutSunday++;
   }   
   double atr = iATR(symbol, tf, 100, shiftWithoutSunday + 10) / 10;
   double gadblSlope = 0.0;
   if ( atr != 0 )
   {
      double dblTma = calcTmaTrue( symbol, tf, shiftWithoutSunday );
      double dblPrev = calcPrevTrue( symbol, tf, shiftWithoutSunday );
      gadblSlope = ( dblTma - dblPrev ) / atr;
   }
   
   return ( gadblSlope );

}

//+------------------------------------------------------------------+
//| calcTmaTrue()                                                                        |
//+------------------------------------------------------------------+
double calcTmaTrue( string symbol, int tf, int inx )
{
   // return ( iMA( symbol, tf, 21, 0, MODE_LWMA, PRICE_CLOSE, inx ) );

   double dblSum  = 0;
   double dblSumw = 0;
   int jnx, knx;
   int sundayCandles = 0;

   for ( jnx = 0, knx = 21; jnx < 21; jnx++, knx-- )
   {
      if ( addSundayToMonday && sundayCandlesDetected && tf == PERIOD_D1 )
      {
         if ( TimeDayOfWeek( iTime( symbol, PERIOD_D1, inx + jnx + sundayCandles ) ) == 0 ) sundayCandles++;
      }   
      dblSum  += iClose( symbol, tf, inx + jnx + sundayCandles ) * knx;
      dblSumw += knx;
   }
   
   return ( dblSum / dblSumw );
}

//+------------------------------------------------------------------+
//| calcPrevTrue()                                                                        |
//+------------------------------------------------------------------+
double calcPrevTrue( string symbol, int tf, int inx )
{
   double dblSum  = iClose( symbol, tf, inx ) * 20;
   double dblSumw = 20;
   int jnx, knx;
   int sundayCandles = 0;
   
   for ( jnx = 1, knx = 21; jnx < 22; jnx++, knx-- )
   {
      if ( addSundayToMonday && sundayCandlesDetected && tf == PERIOD_D1 )
      {
         if ( TimeDayOfWeek( iTime( symbol, PERIOD_D1, inx + jnx + sundayCandles ) ) == 0 ) sundayCandles++;
      }   
      dblSum  += iClose( symbol, tf, inx + jnx + sundayCandles ) * knx;
      dblSumw += knx;
   }
   
   return ( dblSum / dblSumw );
}

